package rition.backend.api.v1.panel;

import org.springframework.web.bind.annotation.*;
import rition.backend.api.v1.dto.request.ContractAddRequest;
import rition.backend.api.v1.dto.response.ContractResponse;
import rition.backend.api.v1.dto.response.Response;
import rition.common.data.dto.service.panel.ContractAddDto;
import rition.common.data.entity.ContractEntity;
import rition.service.panel.ContractService;

import java.util.ArrayList;
import java.util.List;

@RestController
@RequestMapping("/panel/contract")
public class ContractController {
    private final ContractService contractService;

    public ContractController(ContractService contractService) {
        this.contractService = contractService;
    }

    @GetMapping("/list")
    public Response<List<ContractResponse>> getContractList() {
        List<ContractEntity> contractEntityList = contractService.getContractList();
        List<ContractResponse> responseList = new ArrayList<>(contractEntityList.size());
        for (var contractEntity : contractEntityList) {
            ContractResponse response = new ContractResponse();
            response.setId(contractEntity.getId());
            response.setContract(contractEntity.getContract());
            response.setType(contractEntity.getType());
            response.setCreateTime(contractEntity.getCreateTime());

            responseList.add(response);
        }

        return Response.success(responseList);
    }

    @PostMapping("/add")
    public Response<Object> addContract(@RequestBody ContractAddRequest contractAddRequest) {
        ContractAddDto contractAddDto = new ContractAddDto();
        contractAddDto.setContract(contractAddRequest.getContract());
        contractAddDto.setType(contractAddRequest.getType());

        contractService.addContract(contractAddDto);

        return Response.success();
    }

    @PostMapping("/delete")
    public Response<Object> deleteContract(@RequestParam("id") Long contractId) {
        contractService.deleteContract(contractId);

        return Response.success();
    }
}
