package rition.service.panel;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.yitter.idgen.YitIdHelper;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import rition.common.data.dao.mapper.RuleMapper;
import rition.common.data.dto.service.AlertRuleDto;
import rition.common.data.dto.service.panel.AlertRuleAddDto;
import rition.common.data.entity.RuleEntity;
import rition.common.data.enums.Constants;

import java.time.Instant;
import java.util.List;

@Service
public class AlertRuleService {

    private final RuleMapper ruleMapper;
    private final RedisTemplate<String, AlertRuleDto> redisTemplate;

    public AlertRuleService(RuleMapper ruleMapper,
                            RedisTemplate<String, AlertRuleDto> redisTemplate) {
        this.ruleMapper = ruleMapper;
        this.redisTemplate = redisTemplate;
    }

    public void addAlertRule(AlertRuleAddDto alertRuleAddDto) {
        RuleEntity ruleEntity = new RuleEntity();
        ruleEntity.setId(YitIdHelper.nextId());
        ruleEntity.setExpr(alertRuleAddDto.getExpression());
        ruleEntity.setCond(alertRuleAddDto.getCondition());
        ruleEntity.setThreshold(alertRuleAddDto.getThreshold());
        ruleEntity.setTrig(alertRuleAddDto.getTrigger());
        ruleEntity.setComment(alertRuleAddDto.getDescription());

        var now = Instant.now();
        ruleEntity.setCreateTime(now);
        ruleEntity.setUpdateTime(now);
        ruleEntity.setStatus(Constants.EntityCommonStatus.NORMAL);

        ruleMapper.insert(ruleEntity);

        AlertRuleDto alertRuleDto = new AlertRuleDto();
        alertRuleDto.setId(ruleEntity.getId());
        alertRuleDto.setExpression(ruleEntity.getExpr());
        alertRuleDto.setCondition(ruleEntity.getCond());
        alertRuleDto.setThreshold(ruleEntity.getThreshold());
        alertRuleDto.setTrigger(ruleEntity.getTrig());
        alertRuleDto.setDescription(ruleEntity.getComment());
        alertRuleDto.setCreateTime(now);

        // 规则缓存到redis
        redisTemplate.opsForHash().put(Constants.RedisKeys.RULE_CACHE, alertRuleDto.getId().toString(), alertRuleDto);
    }

    public List<RuleEntity> getRule() {
        var query = new LambdaQueryWrapper<RuleEntity>()
                .eq(RuleEntity::getStatus, Constants.EntityCommonStatus.NORMAL);

        return ruleMapper.selectList(query);
    }

    public int deleteRule(Long ruleId) {
        var query = new LambdaUpdateWrapper<RuleEntity>()
                .set(RuleEntity::getStatus, Constants.EntityCommonStatus.DELETED)
                .eq(RuleEntity::getId, ruleId)
                .eq(RuleEntity::getStatus, Constants.EntityCommonStatus.NORMAL);
        var updatedRows = ruleMapper.update(query);
        // 删除相应的缓存
        if (updatedRows != 0) {
            redisTemplate.opsForHash().delete(Constants.RedisKeys.RULE_CACHE, ruleId);
        }

        return updatedRows;
    }
}
