package rition.service.panel;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.yitter.idgen.YitIdHelper;
import org.springframework.stereotype.Service;
import rition.common.data.dao.mapper.ContractMapper;
import rition.common.data.dto.service.panel.ContractAddDto;
import rition.common.data.entity.ContractEntity;
import rition.common.data.enums.Constants;
import rition.common.exception.ServiceException;
import rition.common.exception.code.ServiceCode;

import java.time.Instant;
import java.util.List;

@Service
public class ContractService {

    private final ContractMapper contractMapper;

    public ContractService(ContractMapper contractMapper) {
        this.contractMapper = contractMapper;
    }

    public List<ContractEntity> getContractList() {
        var query = new LambdaQueryWrapper<ContractEntity>()
                .eq(ContractEntity::getStatus, Constants.EntityCommonStatus.NORMAL)
                .last("limit 100");

        return contractMapper.selectList(query);
    }

    private static final int MAX_CONTRACT_NUM = 20;

    public void addContract(ContractAddDto contractAddDto) {
        var countQuery = new LambdaQueryWrapper<ContractEntity>()
                .eq(ContractEntity::getStatus, Constants.EntityCommonStatus.NORMAL);
        var count = contractMapper.selectCount(countQuery);
        if (count > MAX_CONTRACT_NUM) {
            ServiceException.error(ServiceCode.ContractTooManyData);
        }

        var now = Instant.now();
        var entity = ContractEntity.builder()
                .id(YitIdHelper.nextId())
                .contract(contractAddDto.getContract())
                .type(contractAddDto.getType())
                .createTime(now)
                .updateTime(now)
                .status(Constants.EntityCommonStatus.NORMAL)
                .build();

        contractMapper.insert(entity);
    }

    public void deleteContract(Long id) {
        var query = new LambdaUpdateWrapper<ContractEntity>()
                .set(ContractEntity::getStatus, Constants.EntityCommonStatus.DELETED)
                .eq(ContractEntity::getId, id)
                .eq(ContractEntity::getStatus, Constants.EntityCommonStatus.NORMAL);
        contractMapper.update(query);
    }
}
